import type { SimpleGitOptions } from '../types';
import { PluginStore } from './plugin-store';
export declare function customBinaryPlugin(plugins: PluginStore, input?: SimpleGitOptions['binary'], allowUnsafe?: boolean): void;
