import { type StyledChar } from '@alcalzone/ansi-tokenize';
import { type DOMElement } from './dom.js';
import { type Selection } from './selection.js';
type Result = {
    output: string;
    outputHeight: number;
    staticOutput: string;
    styledOutput: StyledChar[][];
    cursorPosition?: {
        row: number;
        col: number;
    };
};
declare const renderer: (node: DOMElement, isScreenReaderEnabled: boolean, selection?: Selection, selectionStyle?: (char: StyledChar) => StyledChar) => Result;
export default renderer;
