import { FetchResult } from '../../../typings';
import { StringTask } from '../types';
import { EmptyTask } from './task';
export declare function fetchTask(remote: string, branch: string, customArgs: string[]): StringTask<FetchResult> | EmptyTask;
