import type { SimpleGitPlugin } from './simple-git-plugin';
import type { SimpleGitPluginConfig } from '../types';
export declare function blockUnsafeOperationsPlugin({ allowUnsafeProtocolOverride, allowUnsafePack, }?: SimpleGitPluginConfig['unsafe']): SimpleGitPlugin<'spawn.args'>;
