/**
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * @format
 */
export type { Config, DirtiedFunction, MeasureFunction, Node, } from './wrapAssembly.ts';
declare const Yoga: import("./wrapAssembly.ts").Yoga;
export default Yoga;
export * from './generated/YGEnums.ts';
