import { PullDetail, PullResult } from '../../../typings';
import { PullFailedSummary } from '../responses/PullSummary';
import { TaskParser } from '../types';
export declare const parsePullDetail: TaskParser<string, PullDetail>;
export declare const parsePullResult: TaskParser<string, PullResult>;
export declare function parsePullErrorResult(stdOut: string, stdErr: string): "" | PullFailedSummary;
