'use strict';

Object.defineProperty(exports, Symbol.toStringTag, { value: 'Module' });

const AbortError = require('./AbortError.js');
const TimeoutError = require('./TimeoutError.js');



exports.AbortError = AbortError.AbortError;
exports.TimeoutError = TimeoutError.TimeoutError;
